==> unicore-ucc6-5.0.0-1.src.tar.gz: ucc-core/src/main/java/de/fzj/unicore/ucc/actions/CreateStorage.java <==
package de.fzj.unicore.ucc.actions;

import java.util.Calendar;
import java.util.List;

import javax.xml.namespace.QName;

import org.apache.commons.cli.OptionBuilder;
import org.unigrids.services.atomic.types.PropertyType;
import org.unigrids.x2006.x04.services.smf.CreateSMSDocument;
import org.unigrids.x2006.x04.services.smf.StorageBackendParametersDocument.StorageBackendParameters;
import org.unigrids.x2006.x04.services.smf.StorageDescriptionType;
import org.unigrids.x2006.x04.services.smf.StorageFactoryPropertiesDocument.StorageFactoryProperties;
import org.w3.x2005.x08.addressing.EndpointReferenceType;

import de.fzj.unicore.uas.StorageFactory;
import de.fzj.unicore.uas.client.StorageClient;
import de.fzj.unicore.uas.client.StorageFactoryClient;
import de.fzj.unicore.ucc.IServiceInfoProvider;
import de.fzj.unicore.ucc.helpers.StorageFactoryLister;
import de.fzj.unicore.ucc.helpers.StorageFactoryLister.StorageFactoryFilter;
import de.fzj.unicore.ucc.util.UnitParser;
import eu.unicore.security.util.client.IClientProperties;

/**
 * creates an SMS instance using the StorageFactory service
 * 
 * @since 1.3.0
 * @author schuller
 */
public class CreateStorage extends ActionBase implements IServiceInfoProvider{

	/**
	 * the initial lifetime (in days) for newly created SMSs
	 */
	private int initialLifeTime;

	/**
	 * factory URL to use
	 */
	private String factoryURL;

	/**
	 * site where to create the storage
	 */
	private String siteName;


	/**
	 * storage type to create
	 */
	private String storageType;

	public static final String OPT_TYPE_LONG="type";
	public static final String OPT_TYPE="t";

	@Override
	@SuppressWarnings("all")
	protected void createOptions() {
		super.createOptions();
		getOptions().addOption(OptionBuilder.withLongOpt(OPT_LIFETIME_LONG)
				.withDescription("Initial lifetime (in days) for created storages.")
				.withArgName("Lifetime")
				.hasArg()
				.isRequired(false)
				.create(OPT_LIFETIME)
				);

		getOptions().addOption(OptionBuilder.withLongOpt(OPT_SITENAME_LONG)
				.withDescription("Name of the site")
				.withArgName("Site")
				.hasArg()
				.isRequired(false)
				.create(OPT_SITENAME)
				);

		getOptions().addOption(OptionBuilder.withLongOpt(OPT_FACTORY_LONG)
				.withDescription("Factory URL")
				.withArgName("Factory")
				.hasArg()
				.isRequired(false)
				.create(OPT_FACTORY)
				);
		getOptions().addOption(OptionBuilder.withLongOpt(OPT_TYPE_LONG)
				.withDescription("Storage type")
				.withArgName("Type")
				.hasArg()
				.isRequired(false)
				.create(OPT_TYPE)
				);
	}

	@Override
	public String getName(){
		return "create-storage";
	}

	@Override
	public String getArgumentList(){
		return "[param1=value1,...]";
	}

	@Override
	public String getSynopsis(){
		return "Creates a storage service instance. " +
				"The StorageFactory to be used can be specified. Optionally, " +
				"type of storage plus any additional parameters can be given.";

	}
	@Override
	public String getDescription(){
		return "create a storage service instance";
	}

	@Override
	public void process() {
		super.process();

		initialLifeTime=getNumericOption(OPT_LIFETIME_LONG, OPT_LIFETIME, -1);
		if(initialLifeTime>0){
			verbose("New SMSs will have a lifetime of <"+initialLifeTime+"> days.");
		}else{
			verbose("Using site default for SMS lifetime.");
		}

		storageType=getOption(OPT_TYPE_LONG, OPT_TYPE);
		factoryURL=getOption(OPT_FACTORY_LONG, OPT_FACTORY);
		StorageFactoryClient sfc=null;

		if(factoryURL==null){
			siteName=getOption(OPT_SITENAME_LONG, OPT_SITENAME);
			if(siteName!=null){
				verbose("Looking for factory at site <"+siteName+">");
				//resolve
				StorageFactoryLister sfl=new StorageFactoryLister(registry,securityProperties,this,new StorageNameFilter());
				List<StorageFactoryClient>list=sfl.list();
				if(list.size()>0){
					sfc=list.get(0);
					factoryURL=sfc.getUrl();
				}
			}
			else{
				verbose("No factory specified, will choose one from registry.");
				try{
					StorageFactoryLister sfl=new StorageFactoryLister(registry,securityProperties,this,new StorageTypeFilter());
					List<StorageFactoryClient>list=sfl.list();
					if(list.size()>0){
						sfc=list.get(0);
						factoryURL=sfc.getUrl();
					}
				}catch(Exception ex){
					error("Can't get service list", ex);
					endProcessing(ERROR);
				}

			}
		}
		else{
			verbose("Using factory at <"+factoryURL+">");
			EndpointReferenceType epr=EndpointReferenceType.Factory.newInstance();
			epr.addNewAddress().setStringValue(factoryURL);
			try{
				sfc=new StorageFactoryClient(factoryURL,epr,securityProperties);
			}
			catch(Exception ex){
				error("Can't create StorageFactory client",ex);
				endProcessing(ERROR);
			}
		}

		if(sfc==null){
			error("No storage factory available!",null);
			endProcessing(ERROR);
		}
		
		try{
			CreateSMSDocument in=CreateSMSDocument.Factory.newInstance();
			in.addNewCreateSMS();
			if(initialLifeTime>0){
				in.getCreateSMS().addNewTerminationTime().setCalendarValue(getTermTime());
			}
			if(storageType!=null){
				verbose("Will create storage of type : "+storageType);
				StorageDescriptionType desc=in.getCreateSMS().addNewStorageDescription();
				desc.setStorageBackendType(storageType);
				if(getCommandLine().getArgs().length>1){
					//other parameters from the cmdline as key=value
					StorageBackendParameters params=desc.addNewStorageBackendParameters();
					for(int i=1;i<getCommandLine().getArgs().length;i++){
						String arg=getCommandLine().getArgs()[i];
						String[]sp=arg.split("=",2);
						PropertyType prop=params.addNewProperty();
						prop.setName(sp[0]);
						prop.setValue(sp[1]);
						verbose("Have parameter : "+arg);
					}
				}
			}
			StorageClient sc=sfc.createSMS(in);
			String addr=sc.getEPR().getAddress().getStringValue();
			message(addr);
			setLastStorageAddress(addr);
		}catch(Exception ex){
			error("Could not create storage",ex);
			endProcessing(ERROR);
		}
	}

	protected Calendar getTermTime(){
		Calendar c = Calendar.getInstance();
		c.add(Calendar.DATE, initialLifeTime);
		return c;
	}

	@Override
	public String getCommandGroup(){
		return "General";
	}

	public QName getQName(){
		return StorageFactory.SMF_PORT;
	}

	public String getServiceName(){
		return "StorageFactory";
	}

	public String getServiceDetails(EndpointReferenceType epr, IClientProperties securityProperties){
		try{
			StorageFactoryClient smf=new StorageFactoryClient(epr.getAddress().getStringValue(),epr,securityProperties);
			StorageFactoryProperties pr=smf.getResourcePropertiesDocument().getStorageFactoryProperties();
			StringBuilder sb=new StringBuilder();
			boolean first=true;
			for(StorageDescriptionType desc: pr.getStorageDescriptionArray()){
				if(!first)sb.append(", ");
				sb.append(getBriefDescription(desc));
				first=false;
			}
			return sb.toString();
		}catch(Exception ex){
			return "N/A ["+getDetailMessage(ex)+"]";
		}
	}

	private UnitParser units = UnitParser.getCapacitiesParser(2);
	protected String getBriefDescription(StorageDescriptionType desc){
		StringBuilder sb=new StringBuilder();
		sb.append(desc.getStorageBackendType());
		try{
			double free=desc.getFileSystem().getDiskSpace().getExactArray()[0].getDoubleValue();
			sb.append(": ").append(units.getHumanReadable(free)).append(" av.");
		}catch(Exception ex){}
		return sb.toString();
	}

	private static String lastStorageAddress;

	public static String getLastStorageAddress() {
		return lastStorageAddress;
	}

	private class StorageNameFilter implements StorageFactoryFilter{
		@Override
		public boolean accept(StorageFactoryClient smf) {
			String url=smf.getEPR().getAddress().getStringValue();
			return url.contains("/"+siteName+"/");
		}
	}

	protected static void setLastStorageAddress(String lastAddress) {
		CreateStorage.lastStorageAddress = lastAddress;
	}

	private class StorageTypeFilter implements StorageFactoryFilter{
		@Override
		public boolean accept(StorageFactoryClient smf) {
			try{
				if(storageType==null)return true;
				
				StorageDescriptionType[]desc=smf.getResourcePropertiesDocument().getStorageFactoryProperties().getStorageDescriptionArray();
				for(StorageDescriptionType d: desc){
					if(storageType.equals(d.getStorageBackendType()))return true;
				}
			}catch(Exception ex){
				error("Error checking factory at <"+smf.getUrl()+">",ex);
			}
			return false;
		}
	}
}

==> unicore-ucc6-6.0.0-0.src.tar.gz: ucc-core/src/main/java/de/fzj/unicore/ucc/actions/CreateStorage.java <==
package de.fzj.unicore.ucc.actions;

import java.util.Calendar;
import java.util.List;

import javax.xml.namespace.QName;

import org.apache.commons.cli.OptionBuilder;
import org.unigrids.services.atomic.types.PropertyType;
import org.unigrids.x2006.x04.services.smf.CreateSMSDocument;
import org.unigrids.x2006.x04.services.smf.StorageBackendParametersDocument.StorageBackendParameters;
import org.unigrids.x2006.x04.services.smf.StorageDescriptionType;
import org.unigrids.x2006.x04.services.smf.StorageFactoryPropertiesDocument.StorageFactoryProperties;
import org.w3.x2005.x08.addressing.EndpointReferenceType;

import de.fzj.unicore.uas.StorageFactory;
import de.fzj.unicore.uas.client.StorageClient;
import de.fzj.unicore.uas.client.StorageFactoryClient;
import de.fzj.unicore.ucc.IServiceInfoProvider;
import de.fzj.unicore.ucc.helpers.StorageFactoryLister;
import de.fzj.unicore.ucc.helpers.StorageFactoryLister.StorageFactoryFilter;
import de.fzj.unicore.ucc.util.UnitParser;
import eu.unicore.util.httpclient.IClientConfiguration;

/**
 * creates an SMS instance using the StorageFactory service
 * 
 * @since 1.3.0
 * @author schuller
 */
public class CreateStorage extends ActionBase implements IServiceInfoProvider{

	/**
	 * the initial lifetime (in days) for newly created SMSs
	 */
	private int initialLifeTime;

	/**
	 * factory URL to use
	 */
	private String factoryURL;

	/**
	 * site where to create the storage
	 */
	private String siteName;

	/**
	 * storage type to create
	 */
	private String storageType;

	public static final String OPT_TYPE_LONG="type";
	public static final String OPT_TYPE="t";

	@Override
	@SuppressWarnings("all")
	protected void createOptions() {
		super.createOptions();
		getOptions().addOption(OptionBuilder.withLongOpt(OPT_LIFETIME_LONG)
				.withDescription("Initial lifetime (in days) for created storages.")
				.withArgName("Lifetime")
				.hasArg()
				.isRequired(false)
				.create(OPT_LIFETIME)
				);

		getOptions().addOption(OptionBuilder.withLongOpt(OPT_SITENAME_LONG)
				.withDescription("Name of the site")
				.withArgName("Site")
				.hasArg()
				.isRequired(false)
				.create(OPT_SITENAME)
				);

		getOptions().addOption(OptionBuilder.withLongOpt(OPT_FACTORY_LONG)
				.withDescription("Factory URL")
				.withArgName("Factory")
				.hasArg()
				.isRequired(false)
				.create(OPT_FACTORY)
				);
		getOptions().addOption(OptionBuilder.withLongOpt(OPT_TYPE_LONG)
				.withDescription("Storage type")
				.withArgName("Type")
				.hasArg()
				.isRequired(false)
				.create(OPT_TYPE)
				);
	}

	@Override
	public String getName(){
		return "create-storage";
	}

	@Override
	public String getArgumentList(){
		return "[param1=value1,...]";
	}

	@Override
	public String getSynopsis(){
		return "Creates a storage service instance. " +
				"The StorageFactory to be used can be specified. Optionally, " +
				"the type of storage plus any additional parameters can be given.";

	}
	@Override
	public String getDescription(){
		return "create a storage service instance";
	}

	@Override
	public void process() {
		super.process();

		initialLifeTime=getNumericOption(OPT_LIFETIME_LONG, OPT_LIFETIME, -1);
		if(initialLifeTime>0){
			verbose("New SMSs will have a lifetime of <"+initialLifeTime+"> days.");
		}else{
			verbose("Using site default for SMS lifetime.");
		}

		storageType=getOption(OPT_TYPE_LONG, OPT_TYPE);
		factoryURL=getOption(OPT_FACTORY_LONG, OPT_FACTORY);
		StorageFactoryClient sfc=null;

		if(factoryURL==null){
			siteName=getOption(OPT_SITENAME_LONG, OPT_SITENAME);
			if(siteName!=null){
				verbose("Looking for factory at site <"+siteName+">");
			}
			else{
				verbose("No factory specified, will choose one from registry.");
			}
		}
		
		//resolve
		StorageFactoryLister sfl=new StorageFactoryLister(registry,securityProperties,this,new Filter());
		List<StorageFactoryClient>list=sfl.list();
		if(list.size()>0){
			sfc=list.get(0);
			factoryURL=sfc.getUrl();
			verbose("Using factory at <"+factoryURL+">");
		}

		if(sfc==null){
			error("No suitable storage factory available!",null);
			endProcessing(ERROR);
		}
		
		try{
			StorageClient sc=sfc.createSMS(getCreateSMSDocument());
			String addr=sc.getEPR().getAddress().getStringValue();
			message(addr);
			setLastStorageAddress(addr);
		}catch(Exception ex){
			error("Could not create storage",ex);
			endProcessing(ERROR);
		}
	}

	protected CreateSMSDocument getCreateSMSDocument(){
		CreateSMSDocument in=CreateSMSDocument.Factory.newInstance();
		in.addNewCreateSMS();
		if(initialLifeTime>0){
			in.getCreateSMS().addNewTerminationTime().setCalendarValue(getTermTime());
		}
		if(storageType!=null){
			verbose("Will create storage of type : "+storageType);
			StorageDescriptionType desc=in.getCreateSMS().addNewStorageDescription();
			desc.setStorageBackendType(storageType);
			if(getCommandLine().getArgs().length>1){
				//other parameters from the cmdline as key=value
				StorageBackendParameters params=desc.addNewStorageBackendParameters();
				for(int i=1;i<getCommandLine().getArgs().length;i++){
					String arg=getCommandLine().getArgs()[i];
					String[]sp=arg.split("=",2);
					PropertyType prop=params.addNewProperty();
					prop.setName(sp[0]);
					prop.setValue(sp[1]);
					verbose("Have parameter : "+arg);
				}
			}
		}
		return in;
	}
	
	protected Calendar getTermTime(){
		Calendar c = Calendar.getInstance();
		c.add(Calendar.DATE, initialLifeTime);
		return c;
	}

	@Override
	public String getCommandGroup(){
		return "General";
	}

	@Override
	public QName getQName(){
		return StorageFactory.SMF_PORT;
	}

	@Override
	public String getServiceName(){
		return "StorageFactory";
	}

	@Override
	public String getServiceDetails(EndpointReferenceType epr, IClientConfiguration securityProperties){
		try{
			StorageFactoryClient smf=new StorageFactoryClient(epr.getAddress().getStringValue(),epr,securityProperties);
			StorageFactoryProperties pr=smf.getResourcePropertiesDocument().getStorageFactoryProperties();
			StringBuilder sb=new StringBuilder();
			boolean first=true;
			for(StorageDescriptionType desc: pr.getStorageDescriptionArray()){
				if(!first)sb.append(", ");
				sb.append(getBriefDescription(desc));
				first=false;
			}
			return sb.toString();
		}catch(Exception ex){
			return "N/A ["+getDetailMessage(ex)+"]";
		}
	}

	private UnitParser units = UnitParser.getCapacitiesParser(2);
	
	protected String getBriefDescription(StorageDescriptionType desc){
		StringBuilder sb=new StringBuilder();
		sb.append(desc.getStorageBackendType());
		try{
			double free=desc.getFileSystem().getDiskSpace().getExactArray()[0].getDoubleValue();
			sb.append(": ").append(units.getHumanReadable(free)).append(" av.");
		}catch(Exception ex){}
		return sb.toString();
	}

	private static String lastStorageAddress;

	public static String getLastStorageAddress() {
		return lastStorageAddress;
	}

	protected static void setLastStorageAddress(String lastAddress) {
		CreateStorage.lastStorageAddress = lastAddress;
	}

	
	private class Filter implements StorageFactoryFilter{
		
		@Override
		public boolean accept(EndpointReferenceType epr) {
			if(factoryURL==null)return true;
			else{
				return epr.getAddress().getStringValue().equalsIgnoreCase(factoryURL);
			}
		}
		
		@Override
		public boolean accept(StorageFactoryClient smf) {
			return checkName(smf) && checkStorageType(smf); 
		}
		
		private boolean checkName(StorageFactoryClient smf) {
			if(siteName==null)return true;
			String url=smf.getEPR().getAddress().getStringValue();
			return url.contains("/"+siteName+"/");
		}
		
		private boolean checkStorageType(StorageFactoryClient smf) {
			try{
				if(storageType==null)return true;
				StorageDescriptionType[]desc=smf.getResourcePropertiesDocument().getStorageFactoryProperties().getStorageDescriptionArray();
				for(StorageDescriptionType d: desc){
					if(storageType.equals(d.getStorageBackendType()))return true;
				}
			}catch(Exception ex){
				error("Error checking factory at <"+smf.getUrl()+">",ex);
			}
			return false;
		}
		
	}

}
